@@ -8,6 +8,7 @@ package List::Util;
 
 use strict;
 use warnings;
+no warnings 'once';
 require Exporter;
 
 our @ISA        = qw(Exporter);
@@ -15,8 +16,11 @@ our @EXPORT_OK  = qw(
   all any first min max minstr maxstr none notall product reduce sum sum0 shuffle uniq uniqnum uniqstr
   pairs unpairs pairkeys pairvalues pairmap pairgrep pairfirst
 );
-our $VERSION    = "1.46_01";
+our $VERSION    = '1.46_03';
 $VERSION    = eval $VERSION;
+BEGIN {
+  *_NEED_TRY_XS = sub(){1};
+}
 
 # List::Util can be upgraded, leaving Scalar::Util and Sub::Util at their
 # pre-split version.  Those versions relied on List::Util to provide
@@ -24,30 +28,57 @@ $VERSION    = eval $VERSION;
 # compiled List::Util code that was left behind by our old version.  For newer
 # versions of those modules (and this module), we need to localize the stashes
 # while loading so the new subs aren't overwritten.
-{
+if (_NEED_TRY_XS && eval {
+  # old Scalar::Util may copy its version from us.  it may also check our
+  # version, so it must be greater than 1.45.
+  local $VERSION
+    = 9999;
   require Scalar::Util;
-  eval { require Sub::Util };
+}) {
+  # if Scalar::Util copied our version, it is 1.11 at the latest
+  if ($Scalar::Util::VERSION && $Scalar::Util::VERSION == 9999) {
+    $Scalar::Util::VERSION = 1.11;
+  }
 
-  # New Scalar::Util does not load List::Util.  If its $VERSION is false (after
-  # loading it), then it's a pre-split version of Scalar::Util loading us, and
-  # it is not fully loaded yet.
+  # $Scalar::Util::VERSION may be false if it is part way through loading and
+  # trying to load List::Util.  This will only be the case for older versions.
   my $scalar_v = $Scalar::Util::VERSION || 0;
-  my $sub_v = $Sub::Util::VERSION || 0;
+  # Sub::Util should always have an accurate version if it loaded
+  my $sub_v = eval { require Sub::Util } ? $Sub::Util::VERSION : 0;
 
   if (
-    ( $scalar_v <= 1.45 )
-    || ( $sub_v && $sub_v <= 1.45 )
+    ( $scalar_v < 1.46 )
+    || ( $sub_v && $sub_v < 1.46 )
   ) {
-    local %Scalar::Util:: if $scalar_v > 1.45;
-    local %Sub::Util::    if $sub_v > 1.45;
-    @Scalar::Util::EXPORT_FAIL = () if $scalar_v <= 1.45;
+    # localize the stashes for newer versions so we don't effect them
+    local %Scalar::Util:: if $scalar_v >= 1.46;
+    local %Sub::Util::    if $sub_v >= 1.46;
+
+    # Try to find the location of the old List::Util module based on the
+    # Scalar::Util or Sub::Util we loaded.  If this is accurate, it will let
+    # XSLoader load the XS code directly.  XSLoader will automatically fall back
+    # to DynaLoader if this is wrong.
+    my $location = '';
+    if (
+      my $old_lu
+        = $scalar_v < 1.46 ? $INC{'Scalar/Util.pm'}
+        : $sub_v < 1.46    ? $INC{'Sub/Util.pm'}
+        : undef
+    ) {
+      $old_lu =~ s/\b(?:Scalar|Sub)\b(.Util\.pm)$/List$1/;
+      $location = qq{#line 1 "$old_lu"\n};
+    }
 
     # localizing the List::Util stash will break XSLoader::load, so we save and
     # clear it manually.  Using this mechanism on Scalar::Util/Sub::Util would
     # break sub names.
     my %list_stash = %List::Util::;
     %List::Util:: = ();
-    my $success = eval {
+
+    # Scalar::Util may have thought it needed PP code, which we may end up
+    # overwriting.  Silence the warnings this may trigger.
+    my $success = eval $location . q{
+      local $^W = 0;
       local $SIG{__DIE__};
       require XSLoader;
       XSLoader::load(__PACKAGE__);
@@ -55,22 +86,37 @@ $VERSION    = eval $VERSION;
     };
     my $e = $@;
     %List::Util:: = %list_stash;
-    if ($scalar_v <= 1.45) {
+
+    # If we loaded old Scalar::Util, it will have populated @EXPORT_FAIL before
+    # the XS was loaded, when no subs existed.  clear it out and repopulate it
+    # now that it should be fully loaded.
+    if ($scalar_v < 1.46) {
       no strict 'refs';
-      push @Scalar::Util::EXPORT_FAIL,
-        grep !defined &{"Scalar::Util::$_"},
-        qw(weaken isweak isvstring);
+      my %exports = map +($_ => 1), @Scalar::Util::EXPORT_OK;
+      @Scalar::Util::EXPORT_FAIL =
+        grep $exports{$_} && !defined &{"Scalar::Util::$_"},
+        qw(weaken isweak dualvar isvstring set_prototype);
     }
-    die $e unless $success;
+    die $e
+      if !$success
+      && (
+        ( $scalar_v >= 1.23_03 && $scalar_v < 1.46)
+        || ($sub_v && $sub_v < 1.46)
+      );
   }
 }
 
-if (eval { require List::Util::XS; } && $List::Util::XS::VERSION > 1.45) {
-  List::Util::XS->import(@EXPORT_OK);
-}
-else {
-  require List::Util::PP;
-  List::Util::PP->import(@EXPORT_OK);
+{
+  my @imports = @EXPORT_OK;
+  if (eval { require List::Util::XS; } && $List::Util::XS::VERSION >= 1.46) {
+    my %xs_imports = map +($_ => 1), @List::Util::XS::EXPORT_OK;
+    List::Util::XS->import(grep $xs_imports{$_}, @imports);
+    @imports = grep !$xs_imports{$_}, @imports;
+  }
+  if (@imports) {
+    require List::Util::PP;
+    List::Util::PP->import(@imports);
+  }
 }
 
 sub import
